from datetime import datetime
from sqlalchemy.orm import Mapped, MappedColumn
from sqlalchemy import Column, String, DateTime, ForeignKey, Enum, UUID, JSON
from cognee.infrastructure.databases.relational import ModelBase

class OperationType(Enum):
    MERGE_DATA = "MERGE_DATA"
    APPEND_DATA = "APPEND_DATA"

class OperationStatus(Enum):
    STARTED = "OPERATION_STARTED"
    IN_PROGRESS = "OPERATION_IN_PROGRESS"
    COMPLETE = "OPERATION_COMPLETE"
    ERROR = "OPERATION_ERROR"
    CANCELLED = "OPERATION_CANCELLED"

class Operation(ModelBase):
    __tablename__ = "operation"

    id = Column(String, primary_key = True)
    status = Column(Enum(OperationStatus))
    operation_type = Column(Enum(OperationType))

    data_id = Column(UUID, ForeignKey("data.id"))
    meta_data: Mapped[dict] = MappedColumn(type_ = JSON)

    created_at = Column(DateTime, default = datetime.utcnow)



import asyncio
from uuid import UUID
from typing import List, Optional
from multiprocessing import Pool
from ..vector_db_interface import VectorDBInterface
from ..models.DataPoint import DataPoint
from ..models.ScoredResult import ScoredResult
from ..embeddings.EmbeddingEngine import EmbeddingEngine


class WeaviateAdapter(VectorDBInterface):
    async_pool: Pool = None
    embedding_engine: EmbeddingEngine = None

    def __init__(self, url: str, api_key: str, embedding_engine: EmbeddingEngine):
        import weaviate
        import weaviate.classes as wvc
      
        self.embedding_engine = embedding_engine

        self.client = weaviate.connect_to_wcs(
            cluster_url=url,
            auth_credentials=weaviate.auth.AuthApiKey(api_key),
            # headers = {
            #     "X-OpenAI-Api-Key": openai_api_key
            # },
            additional_config=wvc.init.AdditionalConfig(timeout=wvc.init.Timeout(init=30))
        )

    async def embed_data(self, data: List[str]) -> List[float]:
        return await self.embedding_engine.embed_text(data)

    async def create_collection(self, collection_name: str):
        import weaviate.classes.config as wvcc

        event_loop = asyncio.get_event_loop()

        def sync_create_collection():
            return self.client.collections.create(
                name=collection_name,
                properties=[
                    wvcc.Property(
                        name="text",
                        data_type=wvcc.DataType.TEXT,
                        skip_vectorization=True
                    )
                ]
            )

        # try:
        result = await event_loop.run_in_executor(None, sync_create_collection)
        # finally:
        #     event_loop.shutdown_executor()

        return result

    def get_collection(self, collection_name: str):
        return self.client.collections.get(collection_name)

    async def create_data_points(self, collection_name: str, data_points: List[DataPoint]):
        from weaviate.classes.data import DataObject

        data_vectors = await self.embed_data(
            list(map(lambda data_point: data_point.get_embeddable_data(), data_points)))

        def convert_to_weaviate_data_points(data_point: DataPoint):
            return DataObject(
                uuid=data_point.id,
                properties=data_point.payload,
                vector=data_vectors[data_points.index(data_point)]
            )

        objects = list(map(convert_to_weaviate_data_points, data_points))

        return self.get_collection(collection_name).data.insert_many(objects)

    async def retrieve(self, collection_name: str, data_id: str):
        def sync_retrieve():
            return self.get_collection(collection_name).query.fetch_object_by_id(UUID(data_id))

        event_loop = asyncio.get_event_loop()

        # try:
        data_point = await event_loop.run_in_executor(None, sync_retrieve)
        # finally:
            # event_loop.shutdown_executor()

        data_point.payload = data_point.properties
        del data_point.properties

        return data_point

    async def search(
            self,
            collection_name: str,
            query_text: Optional[str] = None,
            query_vector: Optional[List[float]] = None,
            limit: int = None,
            with_vector: bool = False
    ):
        import weaviate.classes as wvc

        if query_text is None and query_vector is None:
            raise ValueError("One of query_text or query_vector must be provided!")

        if query_vector is None:
            query_vector = (await self.embed_data([query_text]))[0]

        # def sync_search():
        search_result = self.get_collection(collection_name).query.hybrid(
            query = None,
            vector = query_vector,
            limit = limit,
            include_vector = with_vector,
            return_metadata = wvc.query.MetadataQuery(score=True),
        )

        return [
            ScoredResult(
                id=str(result.uuid),
                payload=result.properties,
                score=float(result.metadata.score)
            ) for result in search_result.objects
        ]

    async def batch_search(self, collection_name: str, query_texts: List[str], limit: int, with_vectors: bool = False):
        def query_search(query_vector):
            return self.search(collection_name, query_vector=query_vector, limit=limit, with_vector=with_vectors)

        return [await query_search(query_vector) for query_vector in await self.embed_data(query_texts)]

    async def prune(self):
        self.client.collections.delete_all()



import asyncio
from uuid import UUID, uuid4
from typing import Union, BinaryIO, List
import cognee.modules.ingestion as ingestion
from cognee.infrastructure import infrastructure_config

class DatasetException(Exception):
    message: str

    def __init__(self, message: str):
        self.message = message


async def add_standalone(
    data: Union[str, BinaryIO, List[Union[str, BinaryIO]]],
    dataset_id: UUID = uuid4(),
    dataset_name: str = None
):
    db_engine = infrastructure_config.get_config()["database_engine"]
    if db_engine.is_db_done is not True:
        await db_engine.ensure_tables()

    if not data:
        raise DatasetException("Data must be provided to cognee.add(data: str)")

    if isinstance(data, list):
        promises = []

        for data_item in data:
            promises.append(add_standalone(data_item, dataset_id, dataset_name))

        results = await asyncio.gather(*promises)

        return results


    if is_data_path(data):
        with open(data.replace("file://", ""), "rb") as file:
            return await add_standalone(file, dataset_id, dataset_name)

    classified_data = ingestion.classify(data)

    data_id = ingestion.identify(classified_data)

    await ingestion.save(dataset_id, dataset_name, data_id, classified_data)

    return dataset_id

    # await ingestion.vectorize(dataset_id, dataset_name, data_id, classified_data)


def is_data_path(data: str) -> bool:
    return False if not isinstance(data, str) else data.startswith("file://")


from typing import Dict, List
from cognee.infrastructure import infrastructure_config

async def resolve_cross_graph_references(nodes_by_layer: Dict):
    results = []

    unique_layers = nodes_by_layer.keys()

    for layer_id, layer_nodes in nodes_by_layer.items():
        # Filter unique_layer_uuids to exclude the current layer
        other_layers = [uuid for uuid in unique_layers if uuid != layer_id]

        for other_layer in other_layers:
            results.append(await get_nodes_by_layer(other_layer, layer_nodes))

    return results

async def get_nodes_by_layer(layer_id: str, layer_nodes: List):
    vector_engine = infrastructure_config.get_config()["vector_engine"]

    score_points = await vector_engine.batch_search(
        layer_id,
        [layer_node["description"] for layer_node in layer_nodes],
        limit = 2
    )

    return {
        "layer_id": layer_id,
        "layer_nodes": connect_score_points_to_node(score_points, layer_nodes)
    }

def connect_score_points_to_node(score_points, layer_nodes):
    return [
        {
            "id": node["node_id"],
            "score_points": score_points[node_index]
        } for node_index, node in enumerate(layer_nodes)
    ]



import logging
from cognee.infrastructure import infrastructure_config
from.extraction.extract_summary import extract_summary

logger = logging.getLogger(__name__)

async def get_content_summary(content: str):
    try:
        return await extract_summary(
            content,
            infrastructure_config.get_config()["summarization_model"]
        )
    except Exception as error:
        logger.error("Error extracting summary from content: %s", error, exc_info = True)
        raise error



import os
import sys
import boto3
from dotenv import load_dotenv

# Get the directory that contains your script
current_dir = os.path.dirname(os.path.abspath(__file__))

# Get the parent directory
parent_dir = os.path.dirname(current_dir)

# Add the parent directory to sys.path
sys.path.insert(0, parent_dir)

environment = os.getenv("AWS_ENV", "dev")


def fetch_secret(secret_name:str, region_name:str, env_file_path:str):
    """Fetch the secret from AWS Secrets Manager and write it to the .env file."""
    print("Initializing session")
    session = boto3.session.Session()
    print("Session initialized")
    client = session.client(service_name="secretsmanager", region_name=region_name)
    print("Client initialized")

    try:
        response = client.get_secret_value(SecretId=secret_name)
    except Exception as e:
        print(f"Error retrieving secret: {e}")
        return f"Error retrieving secret: {e}"

    if "SecretString" in response:
        secret = response["SecretString"]
    else:
        secret = response["SecretBinary"]

    with open(env_file_path, "w") as env_file:
        env_file.write(secret)
        print("Secrets are added to the .env file.")

    if os.path.exists(env_file_path):
        print(f"The .env file is located at: {env_file_path}")
        load_dotenv()
        print("The .env file is loaded.")
    else:
        print(f"The .env file was not found at: {env_file_path}.")


ENV_FILE_PATH = os.path.abspath("../.env")

if os.path.exists(ENV_FILE_PATH):
    # Load default environment variables (.env)
    load_dotenv()
    print("Environment variables are already loaded.")
else:
    fetch_secret(
        f"promethai-{environment}-backend-secretso-promethaijs-dotenv",
        "eu-west-1",
        ENV_FILE_PATH,
    )



from typing import Protocol

class DatabaseEngine(Protocol):
    async def ensure_tables(self):
        pass

    def database_exists(self, db_name: str) -> bool:
        pass

    def create_database(self, db_name: str):
        pass

    def drop_database(self, db_name: str):
        pass

    async def table_exists(self, table_name: str) -> bool:
        pass

    async def create_tables(self):
        pass

    async def create(self, data):
        pass
